@@ -1,4 +1,4 @@
-/* $Id: rapi_context.h 2355 2006-04-07 18:47:20Z voc $ */
+/* $Id: rapi_context.h 3781 2009-06-08 16:43:43Z mark_ellis $ */
 #ifndef __rapi_context_h__
 #define __rapi_context_h__
 
@@ -15,10 +15,12 @@ typedef struct _RapiContext
 	uint32_t result_1;
 	uint32_t result_2;
 	uint32_t last_error;
-    uint32_t rapi_error;
+	uint32_t rapi_error;
 	bool is_initialized;
-    SynceInfo* info;
-    struct rapi_ops_s *rapi_ops;
+	SynceInfo* info;
+	bool own_info;
+	struct rapi_ops_s *rapi_ops;
+	unsigned refcount;
 } RapiContext;
 
 /**
@@ -28,24 +30,36 @@ RapiContext* rapi_context_current();
 
 /**
  * Set current RapiContext
+ * incs refcount of context, and decs refcount
+ * of previous current context
  */
 void rapi_context_set(RapiContext* context);
 
 /**
- * Create new context
+ * Create new context, with refcount of 1
  */
 RapiContext* rapi_context_new();
 
 /**
- * Free context
+ * Increment refcount of context
  */
-void rapi_context_free(RapiContext* context);
+void rapi_context_ref(RapiContext* context);
+
+/**
+ * Decrement refcount of context, free'd when zero
+ */
+void rapi_context_unref(RapiContext* context);
 
 /*
  * Connect to device
  */
 HRESULT rapi_context_connect(RapiContext* context);
 
+/*
+ * Disconnect from device
+ */
+STDAPI rapi_context_disconnect(RapiContext* context);
+
 /**
  * Begin a command
  */
