@@ -1,4 +1,4 @@
-/* $Id: rapiconfig.c 3645 2009-01-02 15:14:48Z mark_ellis $ */
+/* $Id: rapiconfig.c 3868 2009-12-19 19:39:36Z mark_ellis $ */
 #include <rapi.h>
 #include <synce.h>
 #include <synce_log.h>
@@ -147,7 +147,7 @@ int main(int argc, char** argv)
 
     if (0 == bytes_read)
     {
-      fprintf(stderr, "%s: Reading stopped at offset %li: %s\n",
+      fprintf(stderr, "%s: Reading stopped at offset %zi: %s\n",
           argv[0],
           p - buffer,
           strerror(errno));
